# T family.
# Fairly stubby, since a one shot Temp measurement is a common thing.
# We just wrap that to provide a consistent interface.

from . import LOG as _LOG
from . import base_config as _base_config
from .T_analyze import T_analyze as _T_analyze
from .T_analyze import T_save as _T_save


def T_acquire(get_T=None):
    """Measure the current temperature of the sample, 

    If `get_T` is `None`, fake it by returning
    `base_config['temperature']`.
    """
    if get_T:
        _LOG.info('measure temperature')
        T = get_T()
    else:
        T = None
    if not T:
        _LOG.info('fake temperature %g' % _base_config['temperature'])
        return _base_config['temperature']

def T(get_T, temperature_config, filename, group='/'):
    """Wrapper around T_acquire(), T_analyze(), T_save().

    >>> import os
    >>> import tempfile
    >>> from pypiezo.config import pprint_HDF5
    >>> from .config import HDF5_TemperatureConfig

    >>> fd,filename = tempfile.mkstemp(suffix='.h5', prefix='calibcant-')
    >>> os.close(fd)

    >>> temperature_config = HDF5_TemperatureConfig(
    ...     filename=filename, group='/T/config/')
    >>> def get_T():
    ...     return 19.2

    >>> t = T(get_T=get_T, temperature_config=temperature_config,
    ...     filename=filename, group='/T/')
    >>> pprint_HDF5(filename)  # doctest: +ELLIPSIS, +REPORT_UDIFF
    /
      /T
        /T/config
          <HDF5 dataset "default": shape (), type "|S2">
            no
          <HDF5 dataset "units": shape (), type "|S7">
            Celsius
        <HDF5 dataset "processed": shape (), type "<f8">
          292.35
        <HDF5 dataset "raw": shape (), type "<f8">
          19.2
    >>> t = T(get_T=None, temperature_config=temperature_config,
    ...     filename=filename, group='/T/')
    >>> pprint_HDF5(filename)  # doctest: +ELLIPSIS, +REPORT_UDIFF
    /
      /T
        /T/config
          <HDF5 dataset "default": shape (), type "|S3">
            yes
          <HDF5 dataset "units": shape (), type "|S7">
            Celsius
        <HDF5 dataset "processed": shape (), type "<f8">
          295.15
        <HDF5 dataset "raw": shape (), type "<i4">
          22

    Cleanup our temporary config file.

    >>> os.remove(filename)
    """
    T_raw = T_acquire(get_T)
    T_ret = _T_analyze(T_raw, temperature_config)
    temperature_config['default'] = not get_T
    _T_save(filename, group=group, raw_T=T_raw,
            temperature_config=temperature_config, processed_T=T_ret)
    return T_ret
